% Main function for finding closure in images:
%
% Arguments:
%   img_filename    - filename of the image to process (any format readable
%                     by Matlab
%   output_dir      - the output directory for the solutions
%   num_sups        - number of superpixels to extract
%   num_solutions   - number of closure solutions
%   edge_threshold  - Pb edge threshold (smaller values result in more
%                     solutions)
%   sup_method      - The method used to generate superpixels.
%                     Valid values: 'ncuts', 'turbo' (default)
%   Example: ClosureMain(img_filename, '.', 100, 10, 0.05, 'turbo');
%            This would open the image in img_file, extract 100 superpixels
%            using the Turbopixels method, use an edge threshold of 0.05,
%            generate at most 10 solutions and will store them in the
%            current directory.
%
function constructGraph(imgPath,  numSups,  edgeThresh, supAlg)

    use_gpb = true;
    if (ispc)
        % Global Pb is not supported on windows
        use_gpb = false;
    end
    
    img = im2double(imread(imgPath));

    % Compute Pb
    disp('Computing Pb');
    pb_file = [imgPath(1:end-4),'_pb.mat'];
    if (~exist(pb_file, 'file'))
        [pb, theta, tmap] = pbCGTG(img);
        save(pb_file, 'pb', 'theta', 'tmap');
    else
        load(pb_file);
    end

    if (use_gpb)
        disp('Computing gPb');
        %cur_dir = pwd;
        %cd globalPb;
        gpb_file = [imgPath(1:end-4),'_gpb.mat'];
        rsz = 0.5;
        if ( ~exist(gpb_file, 'file'))
            
%             addpath('./util/Closure/globalPb/lib');
            
            [gPb_thin, gPb, maxo] = globalPb(imgPath, 'temp.bmp', rsz);
            save(gpb_file, 'gPb_thin', 'gPb', 'maxo');
        end
       % cd(cur_dir);
        if (exist(gpb_file, 'file'))
            
             disp('Loading  gPb');
            
            load(gpb_file);
            pb = gPb_thin;
        end
    end
    
    disp('Postprocessing Pb results');
    image_data_file = [imgPath(1:end-4),'_image_data.mat'];
    if (~exist(image_data_file, 'file'))
        
        
        image_data = PrecomputeImageData(img, pb, theta, tmap);
        save(image_data_file, 'image_data');
    else
        load(image_data_file);
    end

    disp(sprintf('Computing %d superpixels', numSups));
    superpixels_file = [imgPath(1:end-4),'_num_sups_',num2str(numSups),'.seg'];
    if (~exist(superpixels_file, 'file'))
        switch (supAlg)
            case 'ncuts'
                sup_image = Superpixels_Ncuts_Pb(img, numSups, image_data_file);
            case 'turbo'
               % sup_image = superpixels_pb_jia(img, num_sups, [], [], [], image_data.pb);
                sup_image = superpixels_pb(img, numSups, [], [], [], image_data.pb);
            
            otherwise
                error('Unsupported superpixel method');
        end
        writeSeg(sup_image, superpixels_file);
    else
        sup_image = readSeg(superpixels_file);
    end
    sup_image = CleanSupImage(sup_image);
    
    

    disp('Generating incindent matrix...');
    
    grabSuperpixeInfo(imgPath, sup_image, image_data, 'area',  [],  edgeThresh);
    
   